%Created by J.T.B. Overvelde - 18 April 2012
%Master's thesis - The Moving Noda Approach in Topology Optimization
%http://www.overvelde.com
%
%Initialize problem, nodal distribution and background mesh.

function time1=InitMesh()

GlobalConst

tic %Mesh timer

%Problem definition constants
pCon.Lx=48;                                     %Width of the beam
pCon.Ly=12;                                     %Height of the beam
pCon.P=0.1;                                     %Maximum force in parabolic loading
pCon.E=1;                                       %Elasticiy modulus
pCon.nu=0.3;                                    %Poisson's ratio
pCon.b=[0;0];                                   %Body Force
pCon.D=pCon.E/(1-pCon.nu^2)*[1       pCon.nu 0; %Plain strain elasticy matrix
                             pCon.nu 1       0; 
                             0       0       (1-pCon.nu)/2];

%Meshless discretization constants
mCon.nx=10;                   %number of nodes along the width
mCon.ny=10;                   %number of nodes along the height
mCon.n=mCon.nx*mCon.ny;       %Number of total nodes
mCon.dx=pCon.Lx/(mCon.nx-1);  %Horizontal distance between nodes
mCon.dy=pCon.Ly/(mCon.ny-1);  %Vertical distance between
mCon.d=1.15;                  %Relative smoothing length
mCon.pn=2;                    %Number of monomials in the Monomial basis vector
mCon.dm=[mCon.d*mCon.dx ; mCon.d*mCon.dy];   %Smoothing length in x and y direction, respectively.

%Create nodes
for i=1:mCon.n
    nodes(i).x=[];       %nodes coordinates
    nodes(i).nen=[];     %Neighboring nodes
    nodes(i).BC.type=0;
    nodes(i).BC.len=[];
    nodes(i).BC.dir=[];
    nodes(i).BC.val=[];
    nodes(i).BC.n=[];
    nodes(i).dV=0;
end

%nodes coordinates
for i=1:mCon.nx
    for j=1:mCon.ny
        nodes((i-1)*mCon.ny+j).x=[mCon.dx*(i-1); mCon.dy*(j-1)-pCon.Ly/2];
    end
end

%nodes volume
for i=1:mCon.n
    if sum(i==[1,mCon.ny,mCon.n-mCon.ny+1,mCon.n])==1
        nodes(i).dV=mCon.dx*mCon.dy/4;
    elseif i<mCon.ny
        nodes(i).dV=mCon.dx*mCon.dy/2;
    elseif sum(i==mCon.ny:mCon.ny:mCon.n)==1
        nodes(i).dV=mCon.dx*mCon.dy/2;
    elseif sum(i==2*mCon.ny-1:mCon.ny:mCon.n)==1
        nodes(i).dV=mCon.dx*mCon.dy/2;
    elseif i>mCon.n-mCon.ny+1
        nodes(i).dV=mCon.dx*mCon.dy/2;
    else
        nodes(i).dV=mCon.dx*mCon.dy;
    end
end

%Displacement boundary conditions
for i=1:mCon.ny
    ua=AnalSol(nodes(i).x);
    nodes(i).BC.type=1;
    nodes(i).BC.len=2;
    nodes(i).BC.dir=[1 2];
    nodes(i).BC.val=ua;
end

%Traction Boundary conditions
for i=2:mCon.nx-1
     nodes((i-1)*mCon.ny+1).BC.type=2;
     [ua,ea,sa]=AnalSol(nodes((i-1)*mCon.ny+1).x);
     nodes((i-1)*mCon.ny+1).BC.val=[sa(2),sa(3)];
     nodes((i-1)*mCon.ny+1).BC.n=[0 0 -1; 0 -1 0];
     nodes(i*mCon.ny).BC.type=2;
     [ua,ea,sa]=AnalSol(nodes(i*mCon.ny).x);
     nodes(i*mCon.ny).BC.val=[sa(2),sa(3)];
     nodes(i*mCon.ny).BC.n=[0 0 1; 0 1 0];
end
for i=2:mCon.ny-1
    nodes(mCon.n-(i-1)).BC.type=2;
    [ua,ea,sa]=AnalSol(nodes(mCon.n-(i-1)).x);
    nodes(mCon.n-(i-1)).BC.val=[sa(1),sa(3)];
    nodes(mCon.n-(i-1)).BC.n=[1 0 0; 0 0 1];
end
for i=[mCon.n-mCon.ny+1,mCon.n]
    nodes(i).BC.type=2;
    [ua,ea,sa]=AnalSol(nodes(i).x);
    nodes(i).BC.val=[sa(1),sa(2)];
    nodes(i).BC.n=[1 0 1; 0 1 1];
end

%Find neighbours of nodes
x1=[nodes.x];
nodelabel=1:mCon.n;
for i=1:mCon.n
    nodes(i).nen=nodelabel(and(abs(x1(1,:)-nodes(i).x(1))<mCon.dm(1),abs(x1(2,:)-nodes(i).x(2))<mCon.dm(2)));
end

time1=toc; %Mesh timer
disp([num2str(time1),' seconds to create problem'])
