%Created by J.T.B. Overvelde - 18 April 2012
%Master's thesis - The Moving Noda Approach in Topology Optimization
%http://www.overvelde.com
%
%Determines the stiffness matrix for given nodal distribution and
%background mesh and determines the displacement by solving the system of
%equations for given boundary conditions.

function [ug,sg,ErrorNorm,time1,time2,time3]=MLPG(FindErrorNorm)

GlobalConst

tic %Assembly timer

Phi=zeros(mCon.n,mCon.n);
for i=1:mCon.n
    [phi_i,dphidx_i,dphidy_i]=MLSShape([nodes(nodes(i).nen).x]',nodes(i).x,mCon.dm,mCon.pn);
    nj=0;
    for j=nodes(i).nen
        nj=nj+1;
        Phi(i,j)=phi_i(nj);
    end
end
R=inv(Phi);

%Assembly K matrix and f vector for nodes
K=zeros(2*mCon.n);f=zeros(2*mCon.n,1);
Ks=zeros(2*mCon.n,3*mCon.n); T=zeros(3*mCon.n,2*mCon.n);
for i=1:mCon.n
    f([2*i-1;2*i])=pCon.b;
    [psi_i,dpsidx_i,dpsidy_i]=ColShape([nodes(nodes(i).nen).x]',nodes(i).x,mCon.dm,mCon.pn,R(nodes(i).nen,nodes(i).nen));
    nj=0;
    for j=nodes(i).nen
        nj=nj+1;
        Bij=[dpsidx_i(nj) 0; 0 dpsidy_i(nj); dpsidy_i(nj) dpsidx_i(nj)];
        Ks([2*i-1 2*i],[3*j-2 3*j-1 3*j])=Bij';
        T([3*i-2 3*i-1 3*i],[2*j-1 2*j])=pCon.D*Bij;
    end
end

alpha=1e6; kn1=[]; tr=[];
rep=0;
for i=1:mCon.n
    if nodes(i).BC.type==2
        rep=rep+1;
        nt(2*rep-1:2*rep,3*rep-2:3*rep)=nodes(i).BC.n;
        kn1=[kn1 3*i-2:3*i];
        tr=[tr nodes(i).BC.val];
    end
end
total=1:3*mCon.n;
kn2=setdiff(total,kn1);
Q=inv(eye(length(kn1))+alpha*nt'*nt);

K1=Ks(:,kn1);
K2=Ks(:,kn2);
T1=T(kn1,:);
T2=T(kn2,:);

K=K1*Q*T1+K2*T2;
f=f-alpha*K1*Q*nt'*tr';

time1=toc; %Assembly timer
disp([num2str(time1),' seconds to assemble the matrices'])
tic %Solve timer

%Solve for the displacement constants belonging to the shape functions
numy=[]; BCu=[];
for i=1:mCon.n
    if nodes(i).BC.type==1
        numy=[numy 2*i-2+nodes(i).BC.dir];
        BCu=[BCu nodes(i).BC.val'];
    end
end
total=1:2*mCon.n;
numn=setdiff(total,numy);
f(numn)=f(numn)-K(numn,numy)*BCu';
r=zeros(2*mCon.n,1);
for i=1:length(numy)
    r(numy(i))=BCu(i);
end
r(numn)=K(numn,numn)\f(numn);

time2=toc; %Solve timer
disp([num2str(time2),' seconds to solve the system'])
tic %Find nodal values timer

%Find nodal stress and displacements
ug(:,1)=r(1:2:end-1);
ug(:,2)=r(2:2:end);

sg=zeros(mCon.n,3);
S=zeros(3*mCon.n,1);

S(kn2)=T2*r;
S(kn1)=Q*T1*r+alpha*Q*nt'*tr';

sg(:,1)=S(1:3:end-2)';
sg(:,2)=S(2:3:end-1)';
sg(:,3)=S(3:3:end)';

time3=toc; %Find nodal values timer
disp([num2str(time3),' seconds to find the nodal values'])
tic %Find error norm

ErrorNorm=0;
if FindErrorNorm==1
    ErrorNorm2=0;
    AnEnergy=0;
    Di=inv(pCon.D);
    for i=1:mCon.n
            em=zeros(3,1);
            sm=zeros(3,1);
            sm(:)=sg(i,:); 
            em=Di*sm;
            em(3)=1/2*em(3);       
            [uma,ema,sma]=AnalSol(nodes(i).x);
            ErrorNorm2=ErrorNorm2+(em-ema)'*(sm-sma)*nodes(i).dV;
            AnEnergy=AnEnergy+(ema)'*(sma)*nodes(i).dV;
    end
    ErrorNorm=sqrt(ErrorNorm2)/sqrt(AnEnergy);
end

time4=toc;
disp([num2str(time4),' seconds to find the error norm and compliance'])

