%Created by J.T.B. Overvelde - 18 April 2012
%Master's thesis - The Moving Noda Approach in Topology Optimization
%http://www.overvelde.com
%
%Kernel function used in the MLS shape function for node x at coordinate
%xi.

function [w dwdx dwdy]=WeightTensor(x,xi,dm)

rx = abs(x(1)-xi(1))/dm(1);
ry = abs(x(2)-xi(2))/dm(2);
drdx = -sign(x(1)-xi(1))/dm(1);
drdy = -sign(x(2)-xi(2))/dm(2);
if rx>1
    wrx = 0;
    dwdrx = 0;
elseif rx>0.5
    wrx = 4/3-4*rx+4*rx^2-4/3*rx^3;
    dwdrx = (-4+8*rx-4*rx^2)*drdx;
else
    wrx = 2/3-4*rx^2+4*rx^3;
    dwdrx = (-8*rx+12*rx^2)*drdx;
end
if ry>1
    wry = 0;
    dwdry = 0;
elseif ry>0.5
    wry = 4/3-4*ry+4*ry^2-4/3*ry^3;
    dwdry = (-4+8*ry-4*ry^2)*drdy;
else
    wry = 2/3-4*ry^2+4*ry^3;
    dwdry = (-8*ry+12*ry^2)*drdy;
end
w=wrx*wry;
dwdx=wry*dwdrx;
dwdy=wrx*dwdry;

